function [vertexids0, vertexids1, figout]= setCurve(mesh0, mesh1, figin)
    if nargin == 2
        figout = figure;
        subplot(1,2,1);
        trimesh(mesh0.T, mesh0.X(:,1), mesh0.X(:,2), mesh0.X(:,3));
        view([0 90]);
        title('mesh 1');
        axis equal;
        hold on;
        
        subplot(1,2,2);
        trimesh(mesh1.T, mesh1.X(:,1), mesh1.X(:,2), mesh1.X(:,3));
        view([0 90]);
        title('mesh 2');
        axis equal;
        hold on;
     else
        figout = figure(figin); 
    end
    cameratoolbar;
    
    %set curve on mesh 1
    idx00 = selectPoint(figout, mesh0);
    idx01 = selectPoint(figout, mesh0);
    vertexids0 = shortestPathEuklidean(mesh0, idx00, idx01);%shortestCurvatureTensor(mesh0, idx00, idx01);      
    figure(figout);
    subplot(1,4,1);
    X = mesh0.X(vertexids0, :);
    plot3(X(:,1), X(:,2), X(:,3));
    hold on;
    
    %set curve on mesh 1
    idx10 = selectPoint(figout, mesh1);
    idx11 = selectPoint(figout, mesh1); 
    vertexids1 = shortestPathEuklidean(mesh1,idx10, idx11);%shortestCurvatureTensor(mesh1,idx10, idx11);
    figure(figout);
    subplot(1,4,2);
    X = mesh1.X(vertexids1, :);
    plot3(X(:,1), X(:,2), X(:,3));
    hold on; 
end



function vertexids = shortestPathEuklidean(mesh, id0, id1)
    X1 = mesh.X(mesh.E2V(:,1),:);
    X2 = mesh.X(mesh.E2V(:,2),:);
    dist = X1 - X2;
    norms = zeros(size(dist, 1), 1);
    for i=1:size(dist,1)
       norms(i) = norm(dist(i,:));
    end
    
    G = graph(mesh.E2V(:,1)',mesh.E2V(:,2)', norms');
    
    [vertexids, ~] = shortestpath(G,id0, id1);
end

function vertexids = shortestPathCurvature(mesh, id0, id1)
    X1 = abs(mesh.Cmax(mesh.E2V(:,1),:));
    X2 = abs(mesh.Cmax(mesh.E2V(:,2),:));
    curvatureM = (X1 + X2);
    maxC = max(curvatureM);
    curvature = repmat(maxC, size(curvatureM));
    
    X1 = mesh.X(mesh.E2V(:,1),:);
    X2 = mesh.X(mesh.E2V(:,2),:);
    dist = X1 - X2; 
    norm = repmat((sum(dist.^2, 2)).^0.5, 1, 3);
    ndist = dist./norm;
    Cdir = mesh.Umin(mesh.E2V(:,1),:);
    z = sum(ndist.*(Cdir),2);
    curvatureDev = acosd(z);
    indices = find(curvatureDev > 90);
    curvatureDev(indices) = 180-curvatureDev(indices);
    G = graph(mesh.E2V(:,1)',mesh.E2V(:,2)', (curvatureDev').*(curvature'));
    
    [vertexids, ~] = shortestpath(G,id0, id1);
end


function vertexids = shortestPathGeodesic(mesh, id0, id1)
    options.method='discrete';
    [D,S,Q] = perform_fast_marching_mesh(mesh.X, mesh.T, id0, options);
    [path, vertexids, plist] = compute_geodesic_mesh(D,mesh.X, mesh.T, id1, options);
end