function mesh = MeshInfo(X, T, numEigs, areaWeigths)

mesh.X = X;
mesh.T = T;
[mesh.E2V, mesh.T2E, mesh.E2T, mesh.T2T] = connectivity(mesh.T);

mesh.nf = size(mesh.T,1);
mesh.nv = size(mesh.X,1);
mesh.ne = size(mesh.E2V,1);
numEigs = min(numEigs, mesh.nv);

% Normals and areas
mesh.normal = cross(mesh.X(mesh.T(:,1),:) - mesh.X(mesh.T(:,2),:), mesh.X(mesh.T(:,1),:) - mesh.X(mesh.T(:,3),:));
mesh.area = sqrt(sum(mesh.normal.^2, 2))/2;
mesh.normal = mesh.normal./repmat(sqrt(sum(mesh.normal.^2, 2)), [1, 3]);

mesh.sqrt_area = sqrt(sum(mesh.area));

A = sparse(mesh.T, repmat((1:mesh.nf)', [3,1]), repmat(mesh.area, [3,1]), mesh.nv, mesh.nf);
mesh.Nv = A*mesh.normal;
mesh.Nv = mesh.Nv./repmat(sqrt(sum(mesh.Nv.^2,2)), [1,3]);

% Edge length
mesh.SqEdgeLength = sum((mesh.X(mesh.E2V(:,1),:) - mesh.X(mesh.E2V(:,2),:)).^2, 2);

% Eigenstuff
% [mesh.cotLaplacian, mesh.Av] = cotLaplacian(mesh.X, mesh.T);
[mesh.cotLaplacian,~] = per_edge_laplacian(abs(mesh.T2E), mesh.E2V, mesh.area, mesh.SqEdgeLength);
mesh.cotLaplacian = -mesh.cotLaplacian;
[mesh.Ae,~] = per_edge_area(mesh.E2T, mesh.E2V, mesh.area);
areaweights = 1/sum(areaWeigths)*areaWeigths;

try
    [mesh.laplaceBasis, mesh.eigenvalues] = eigs(mesh.cotLaplacian, mesh.Ae, numEigs, 1e-5);
catch
    % In case of trouble make the laplacian definite
    [mesh.laplaceBasis, mesh.eigenvalues] = eigs(mesh.cotLaplacian - 1e-8*speye(mesh.nv), mesh.Ae, numEigs, 'sm');
end
%TODO: what happens if we use the complete basis, no permutation to debug
%the basis representation?
%[mesh.laplaceBasis, mesh.eigenvalues] = eigs(mesh.cotLaplacian - 1e-8*speye(mesh.nv),  spdiags(areaWeigths,0,length(areaWeigths), length(areaWeigths)), min([lastOne, numEigs]), 'sm');

mesh.eigenvalues = diag(mesh.eigenvalues);

id = ones(mesh.nv);
PM = spdiags(id, 0 ,mesh.nv, mesh.nv);
% for i=1:mesh.nv
%     PM(i,i)=1;
% end
q =[];
for i=1:mesh.nv
    if areaweights(i)<=1e-8
        q = [q,i];  
    elseif (size(q, 2)>0) && (areaweights(i) >= 1e-8)
        new_row = q(1,1);
        q = q(2:end);
        row1 = PM(new_row, :);
        row2 = PM(i,:);
        PM(new_row, :) = row2;
        PM(i, :) = row1;
        q = [q, i];
    end
end

% for i=1:mesh.nv
%     isOne = 0;
%     for j=1:mesh.nv
%         if PM(i,j) == 1
%             isOne=isOne+1;
%         end
%     end
%     if isOne ~= 1
%         %PM(i,i) = 1;
%         fprintf('warning row is %d )\n', isOne); toc;
%     end
% end

isOne = sum(PM==1,2);
if sum(isOne~=1)>0
    fprintf('warning row is %d )\n', isOne); toc;
end
    
    
    
    
newweights = PM*areaweights;
newLaplacian =  PM*mesh.cotLaplacian*(PM');
col1 = newLaplacian(:,1);
coln = newLaplacian(:,end);
row1 = newLaplacian(1,:);
rown = newLaplacian(end,:);
nextZero = q(1,1);
lastOne = nextZero-1;
newweights = newweights(1:lastOne);
zeroEntries = size(areaweights,1)-lastOne;
blocksL = mat2cell(newLaplacian, [lastOne, zeroEntries], [lastOne, zeroEntries]);

% fileID = fopen('./../../Laplace.txt','w');
% fprintf(fileID,'%s \n',full(newLaplacian ));
% fclose(fileID);

% pinverseL22 = blocksL{2,2}'*blocksL{2,2};
%determ = det(blocksL{2,2});
%fprintf('determinant %d )\n', determ); toc;
%rankL22 = rank(full(blocksL{2,2}));
%fprintf('rank %d )\n', rankL22); toc;

%inverseL22 =inv(blocksL{2,2});
fprintf("a");
Lgen = blocksL{1,1} - blocksL{1,2}*(blocksL{2,2}\blocksL{2,1});
fprintf("b");
[featureBasis, mesh.genEigenvalues] = eigs(Lgen, spdiags(newweights,0,length(newweights), length(newweights)), min([lastOne, numEigs]), 'SM');
%[featureBasis, mesh.genEigenvalues] = eigs(Lgen, min([lastOne, numEigs]), 'sm');
fprintf("d");
f2 = -1*(blocksL{2,2}\blocksL{2,1})*featureBasis;
fprintf("c");
mesh.featureBasis = (PM')*[featureBasis; f2];

mesh.laplaceBasis = mesh.featureBasis;

%mesh.eigenvalues = (PM')*[diag(mesh.genEigenvalues); zeros(size(f2,1),1)];

%TODO: what happens if we use the complete basis, no permutation to debug
%the basis representation?
%[featureBasis, mesh.genEigenvalues] = eigs(mesh.cotLaplacian - 1e-8*speye(mesh.nv),  +  1e-8*speye(mesh.nv) + spdiags(areaWeigths,0,length(areaWeigths ), length(areaWeigths)), min([lastOne, numEigs]), 'sm');



% try
% [mesh.featureBasis,mesh.genEigenvalues] = eigs(mesh.cotLaplacian,spdiags(areaweights)+1e-8*speye(mesh.nv), numEigs, 'sm');
% catch
%     % In case of trouble make the laplacian definite
%     [mesh.featureBasis,mesh.genEigenvalues] = eigs(mesh.cotLaplacian- 1e-8*speye(mesh.nv), diag(areaweights)+1e-8*speye(mesh.nv), numEigs, 'sm');
% end

% try
%     [mesh.laplaceBasis, mesh.eigenvalues] = eigs(mesh.cotLaplacian, diag(areaweights)+1e-8*speye(mesh.nv), numEigs, 'sm');
% catch
%     % In case of trouble make the laplacian definite
%     [mesh.laplaceBasis, mesh.eigenvalues] = eigs(mesh.cotLaplacian - 1e-8*speye(mesh.nv), diag(areaweights)+1e-8*speye(mesh.nv), numEigs, 'sm');
% end
% mesh.eigenvalues = diag(mesh.eigenvalues);

end
