% Computes the DFT of the functions on the curves with id curveid
% F = Fourier coefficients
% D = DFT operator
% P = projection matrix that projects functions on the surface to functions
% C = Vertex ids of the curve
% on the curve
% mesh: needs property X containing the vertices
% curves: matrix [curve_id, ordered vertex id]
% mesh_fct: contains the functions defined on the mesh

function [F, D, P, C] = fourierBasis(mesh, curves, curveid, mesh_fct, n_samples, flip, plotnum)
    if nargin <5
        n_samples=50;
    end
    if nargin <6
        flip = false;
    end
    if nargin <7
        plotnum = 1;
    end
    
    if flip
        curvesf = flipud(curves);
    else
        curvesf = curves;
    end
   firstColumn = curvesf(:,1);
   curvei = curvesf(firstColumn==curveid, :); 
   curve = curvei(:,2);
   curve = curve+1;
   C = curve;
   f_C = mesh_fct(curve, :);
   c_X = mesh.X(curve, :);
   c_X_shift = circshift(c_X,1);
   c_X_shift(1,:)=c_X(1,:);
   dist = c_X - c_X_shift;
   %norms = vecnorm(dist');
   for i=1:size(dist,1)
       norms(i) = norm(dist(i,:));
   end
   arc_length = sum(norms);
      
   stepsize = arc_length/n_samples;
   
   f_C_sampled = zeros(n_samples, size(f_C,2));
   f_C_sampled(1,:)=f_C(1,:);
   f_C_sampled(n_samples,:)=f_C(size(f_C,1),:);
   k=0;
   P = sparse(n_samples, size(mesh.X, 1));
   P(1,curve(1))=1;
   P(n_samples,curve(length(curve))) = 1;
   for i=2:n_samples-1
       l = stepsize*(i-1); 
       set_k = false;
       while ~set_k
           if sum(norms(1:k))<l
               k = k+1;
           else
               set_k = true;
               k = k-1;
           end
       end
       
       l_ = l-sum(norms(1:k));
       l_Xk = norm(c_X(k,:) - c_X(k+1,:));
       alpha = l_/l_Xk;
       f_C_sampled(i,:) = (1-alpha)*f_C(k,:) + alpha*f_C(k+1,:);
       P(i, curve(k))=1-alpha;
       P(i, curve(k+1))=alpha;
   end
   %F = fft(f_C_sampled);
%    x=[];
%    sums = 0;
%    for i=1:size(f_C,1)
%        sums = sums+norms(i);
%        x=[x;sums];
%    end
%    figure;
%    for i=1:1000
%        y=f_C(:, i);
%        plot(x,y);
%        hold on;
%    end

%    subplot(2,3,plotnum);
%    x=linspace(1, n_samples, n_samples);
%    for i=1:200
%        y=f_C_sampled(:, i);
%        plot(x,y);
%        hold on;
%    end
%    figure;
%    subplot(2,3,plotnum);
%    x=linspace(1, n_samples, n_samples);
%    fs = P*mesh_fct;
%    for i=201:1400
%        y=fs(:, i);
%        plot(x,y);
%        hold on;
%    end
   
   %compute P
%    for i=1:length(curve)
%        P(i,curve(i)) = 1;
%    end
   %compute DFT operator D
   k=n_samples;
   N=n_samples;
%    D = zeros(k-1,N);
%    for k_=2:k
%        for n=1:N
%            D(k_-1,n)=exp(-1i*2*pi*(k_-1)*(n-1)/N);
%        end
%    end
 
 % uncomment to add zero padding of the curves
 %   N=3*n_samples;
 %Pz = sparse(size(P,1), size(P,2));
 %P = [Pz; P; Pz];
 %end padding
 
 D = zeros(k,N);
   for k_=1:k
       for n=1:N
           D(k_,n)=exp(-1i*2*pi*(k_-1)*(n-1)/N);
       end
   end
   
   %TODO: test if the projection+operator is the same as the fft
   F = D*P*mesh_fct;
   %figure;
   %imagesc(real(F_2-F));
   %figure;
   %imagesc(imag(F_2-F));
   
   
end